function X = ndhelper(fn, X, dim)
%NDHELPER run a function for columns of a matrix on any nd-array
%
% Usage:
%     X = ndhelper(fn, X[, dim])
%
% fn should take a matrix and *always* perform its operation on the columns,
% returning a row vector.
% 
% NDHELPER calls fn so that the operation is applied along dim.
% If dim is unspecified, the first non-singleton dimension. A flaky behavior
% that is pervasive in Matlab-land, so I'm sticking with it. It is recommended
% that you always specify dim.

% Iain Murray, November 2008

if ~exist('dim', 'var')
    dim = find(size(X)~=1, 1); % first non-singleton dimension (can be zero)
    if isempty(dim)
        dim = 2; % For histc Matlab sets to 2.
    end
end

% Arrange into a matrix for fn()
nd = max(dim, ndims(X));
if (nd > 2) || (dim ~= 1)
    perm = [dim:nd 1:(dim-1)];
    % The Matlab help explicitly suggests using shiftdim, but I couldn't cover all
    % cases correctly with that alone, so I went for permute/ipermute.
    X = permute(X, perm);
    sz = size(X);
    X = reshape(X, sz(1), prod(sz(2:end)));
end

X = fn(X);

% Turn back into an ND-array.
if (nd > 2) || (dim ~= 1)
    sz(1) = 1;
    X = reshape(X, sz);
    X = ipermute(X, perm);
end
