function X = sumlog(X, varargin)
%SUMLOG performs sum(log(.)) fast
%
%     X = sumlog(X[, dim]);
% Gives an answer very close (up to some round-off errors) to:
%     X = sum(log(X)[, dim]);
% But faster. And without possible {under,over}flow issues of
%     X = log(prod(X[, dim]));
%
% This routine should give similar answers to sum(reallog(X)), although there
% will be small differences in round-off errors. There may not be a speedup if X
% contains many zeros, NaNs, Infs, or negative numbers. These cases should be
% dealt with correctly, but these cases are not optimized.
%
% (Test carefully however, no responsibility for accuracy is taken by the author.)

% Iain Murray, November 2008

if (ndims(X) == 2) && ((nargin == 2) && (varargin{1} == 1))
    X = sumlog_base(X);
else
    X = ndhelper(@sumlog_base, X, varargin{:});
end
