a = rand(2,5,7,9);
testclose(sum(log(a)), sumlog(a));
for dim = 1:6
    testclose(sum(log(a), dim), sumlog(a, dim));
end

a = rand(2,0,7,9);
testclose(sum(log(a)), sumlog(a));
for dim = 1:6
    testclose(sum(log(a), dim), sumlog(a, dim));
end

a = rand(0,0,7,9);
testclose(sum(log(a)), sumlog(a));
for dim = 1:6
    testclose(sum(log(a), dim), sumlog(a, dim));
end

a = rand(0,1,0,7,9);
testclose(sum(log(a)), sumlog(a));
for dim = 1:6
    testclose(sum(log(a), dim), sumlog(a, dim));
end


a = rand(1,0,7,9);
testclose(sum(log(a)), sumlog(a));
for dim = 1:6
    testclose(sum(log(a), dim), sumlog(a, dim));
end

a = rand(1,1,1,9);
testclose(sum(log(a)), sumlog(a));
for dim = 1:6
    testclose(sum(log(a), dim), sumlog(a, dim));
end

a = rand(1e5,1e1);
testclose(sum(log(a)), sumlog(a));
for dim = 1:3
    testclose(sum(log(a), dim), sumlog(a, dim));
end

% Extra tests that careful version should pass
a = [1; Inf; 1; 3];
testequal(sum(log(a)), sumlog(a));
a = [1; NaN; 1; 3];
testequal(sum(log(a)), sumlog(a));
try
    ok = false;
    a = [1; -Inf; 1; 3];
    sumlog(a);
catch
    ok = true;
end
assert(ok);
a = [1; 1; 0; 3];
testequal(sum(log(a)), sumlog(a));
a = [1; Inf; 0; 3];
testequal(sum(log(a)), sumlog(a));
a = [1; Inf; NaN; 3];
testequal(sum(log(a)), sumlog(a));
a = [1; Inf; NaN; 0];
testequal(sum(log(a)), sumlog(a));
a = [1; Inf; NaN; 0];
testequal(sum(log(a)), sumlog(a));
a = [1; 2; -NaN; 1];
testequal(sum(log(a)), sumlog(a));

% Numerical extremes, including denormal numbers:
a = [realmin; realmax; 1e-320; 1];
testclose(sum(log(a)), sumlog(a));

% This test fails if BATCH_SIZE=1024 as prod overflows
% BATCH_SIZE must be 1023 or less.
u = [ones(1023,1); repmat((2-eps), 1025, 1)];
testclose(sumlog(u), sum(log(u)));


%%%%%%
disp('Now testing singles');
testclosef = @(a,b) testclose(a,b,1e-5); % Less stringent for floats


a = single(rand(2,5,7,9));
testclosef(sum(log(a)), sumlog(a));
for dim = 1:6
    testclosef(sum(log(a), dim), sumlog(a, dim));
end

a = single(rand(2,0,7,9));
testclosef(sum(log(a)), sumlog(a));
for dim = 1:6
    testclosef(sum(log(a), dim), sumlog(a, dim));
end

a = single(rand(0,0,7,9));
testclosef(sum(log(a)), sumlog(a));
for dim = 1:6
    testclosef(sum(log(a), dim), sumlog(a, dim));
end

a = single(rand(0,1,0,7,9));
testclosef(sum(log(a)), sumlog(a));
for dim = 1:6
    testclosef(sum(log(a), dim), sumlog(a, dim));
end


a = single(rand(1,0,7,9));
testclosef(sum(log(a)), sumlog(a));
for dim = 1:6
    testclosef(sum(log(a), dim), sumlog(a, dim));
end

a = single(rand(1,1,1,9));
testclosef(sum(log(a)), sumlog(a));
for dim = 1:6
    testclosef(sum(log(a), dim), sumlog(a, dim));
end

a = single(rand(1e5,1e1));
testclosef(sum(log(a)), sumlog(a));
for dim = 1:3
    testclosef(sum(log(a), dim), sumlog(a, dim));
end

% Extra tests that careful version should pass
a = single([1; Inf; 1; 3]);
testequal(sum(log(a)), sumlog(a));
a = single([1; NaN; 1; 3]);
testequal(sum(log(a)), sumlog(a));
try
    ok = false;
    a = single([1; -Inf; 1; 3]);
    sumlog(a);
catch
    ok = true;
end
assert(ok);
a = single([1; 1; 0; 3]);
testequal(sum(log(a)), sumlog(a));
a = single([1; Inf; 0; 3]);
testequal(sum(log(a)), sumlog(a));
a = single([1; Inf; NaN; 3]);
testequal(sum(log(a)), sumlog(a));
a = single([1; Inf; NaN; 0]);
testequal(sum(log(a)), sumlog(a));
a = single([1; Inf; NaN; 0]);
testequal(sum(log(a)), sumlog(a));
a = single([1; 2; -NaN; 1]);
testequal(sum(log(a)), sumlog(a));

% Numerical extremes, including denormal numbers:
a = single([realmin('single'); realmax('single'); 1e-130; 1]);
testclosef(sum(log(a)), sumlog(a));

% This test fails if BATCH_SIZE=128 as prod overflows
% BATCH_SIZE must be 127 or less.
u = single([ones(127,1); repmat((2-eps('single')), 129, 1)]);
testclosef(sumlog(u), sum(log(u)));
% Not that round-off error for this pathological case is getting quite large!


fprintf('\n\nI should think about single test cases more.\n');
fprintf('Round-off errors are more dangerous here, and quite noticable wth sum(log()) too.\n');
u = single(rand(1e5,1));
fprintf('Example sum(log(x)) roundoff error: %g\n', abs(sum(log(u)) - sum(log(double(u)))));
fprintf('Example sumlog(x) roundoff error: %g\n', abs(sumlog(u) - sum(log(double(u)))));
fprintf('The ordering of the too methods is not always this way around however.\n\n');
